/* wrtilty.c
 *
 * Code to control a motor and a servo using an ATTiny2313. The ATTiny receives
 * commands through the UART and sets its PWM or servo pulse accordingly.
 *
 * John Boiles
 * 31 October 2008 
 */

#include <inttypes.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/sleep.h>
#include "uart.c"

#define OC0 PB2
#define OCR OCR0A
#define DDROC DDRB
#define TIMER0_TOP 255	/* 8-bit PWM */
//Set COM0A1:0 = 2 for non-inverted pwm and 3 for inverted pwm
//Use the following line for FastPWM
//#  define TIMER0_PWM_INIT _BV(WGM00) | _BV(WGM01) | _BV(COM0A1)
//Use the following line for PhaseCorrectPWM
#define TIMER0_PWM_INIT _BV(WGM00) | _BV(COM0A1)
//The clocksource determines the timer's frequency
//CS = 1 ClkIO = Clk
//CS = 2 ClkIO = Clk / 8
//CS = 3 ClkIO = Clk / 64
//CS = 4 ClkIO = Clk / 256
//CS = 5 ClkIO = Clk / 1024//CS = 6,7 External Clock
#define TIMER0_CLOCKSOURCE _BV(CS02) //| _BV(CS00)

//For the servo
#define TIMER1_CLOCKSOURCE _BV(CS11) //Clock / 8

//This should work if it's not volatile, but it didn't
volatile unsigned short ServoPulse;
//Create a 20ms period for servo control
ISR (TIMER1_COMPA_vect)
{
  //PORTB |= 0x08;
  PORTB &= ~0x08;
  OCR1A = TCNT1 + 20000;   //20000 * 1MHz = 20ms
  OCR1B = TCNT1 + ServoPulse;
  //Enable OC
  //TIMSK |= _BV(OCIE1B);
}

//Turn off servo control pulse
ISR (TIMER1_COMPB_vect)
{
  //PORTB &= ~0x08;       //PB3 is servo control
  PORTB |= 0x08;

  //Disable OC
  //TIMSK &= ~(1<<OCIE1B);
}

void SetServo(unsigned char input){
  unsigned short calc;
  calc = 988 + 4*input;
  ServoPulse = calc;
}

void
ioinit (void)                   /* Note [6] */
{
  /* Timer 0 is 8-bit PWM . */
  TCCR0A = TIMER0_PWM_INIT;
  TCCR1A = 0;
  /*
   * Start timer 0.
   *
   * NB: TCCR0A and TCCR0B could actually be the same register, so
   * take care to not clobber it.
   */
  TCCR0B |= TIMER0_CLOCKSOURCE;
  TCCR1B |= TIMER1_CLOCKSOURCE;
  
  /* Set PWM value to 0. */
  OCR = 0;
  
  DDRB |= 0x0F;
  //Set Direction:
  PORTB |= 0x01;
  PORTB &= ~0x02;

  //For inverted servo
  PORTB |= 0x08;

  /* Enable OC1 as output. */
  DDROC |= _BV (OC0);
  
  
  /* Enable timer 1 OC interrupt. */
  TIMSK |= _BV(OCIE1A) | _BV(OCIE1B);
  ServoPulse = 1500;
  sei ();
}



int
main (void)
{

  unsigned char in;
  UART_Init(38400); 
  ioinit ();

  /* loop forever, the interrupts are doing the rest */
  
  while (1){                    
    //sleep_mode();
    in = UART_InChar();
    switch(in){
    case ('a'):
      OCR = UART_InChar();
      PORTB |= 0x01;
      PORTB &= ~0x02;
      break;
    case ('A'):
      in = UART_InChar();
      OCR = in;
      PORTB &= ~0x01;
      PORTB |= 0x02;
      break;
    case ('s'):
      SetServo(UART_InChar());
      break;
    }
  }
  return (0);
}
